#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: 深圳星河软通科技有限公司 A.Star
# @contact: astar@snowland.ltd
# @site: www.astar.ltd
# @file: analytic_geometry_base .py
# @time: 2020/6/5 4:24
# @Software: PyCharm


from abc import ABC

__all__ = [
    'Line',
    'Surface'
]


class Line(ABC):
    pass


class Surface(ABC):
    pass
