# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: 深圳星河软通科技有限公司 A.Star
# @contact: astar@snowland.ltd
# @site: www.astar.ltd
# @file: 
# @time: 
# @Software: PyCharm


from pencildraw import pencil_drawing
from pencildraw.pencils import pencil0, pencil1,pencil2, pencil3, pencil4
from snowland.image.style_migration.img_to_cartoon import cartoonise
from snowland.image.style_migration.img_to_freehand import freehand
from snowland.image.style_migration.img_to_oilpaint import oilpaint
from snowland.image.style_migration.img_to_sketch import sketch
