# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: 深圳星河软通科技有限公司 A.Star
# @contact: astar@snowland.ltd
# @site: www.astar.ltd
# @file: __init__.py
# @time: 2019/8/12 20:47
# @Software: PyCharm


from snowland.graphics.solid_geometry._ploygon import in_polygon
